"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.autoObserverPlugin = void 0;
const autoObserverPlugin = function (babel) {
  const t = babel.types;
  return {
    name: "babel-plugin-mobx-observer-on-every-react-component",
    visitor: {
      Program(path, state) {
        if (isInNodeModules(state)) {
          return;
        }
        let hasReactComponent = false;
        path.traverse({
          "ArrowFunctionExpression|FunctionDeclaration|FunctionExpression|ClassDeclaration|ClassExpression"(path) {
            if (isReactComponent(path)) {
              hasReactComponent = true;
              // Check to see if we have imported observer from mobx-react
              const hasObserverImport = path.hub.file.code.includes('import { observer } from "mobx-react"');
              if (!hasObserverImport) {
                console.log("no observer import in this file, adding one", path.hub.file.opts.filename);
                // Add the import statement
                const importStatement = t.importDeclaration([t.importSpecifier(t.identifier("observer"), t.identifier("observer"))], t.stringLiteral("mobx-react"));
                path.hub.file.path.node.body.unshift(importStatement);
              } else {
                console.log("has observer import in this file", path.hub.file.opts.filename);
              }
            }
          }
        });
        if (!hasReactComponent) {
          console.log("no react components in this file", path.hub.file.opts.filename);
        }
      },
      ArrowFunctionExpression(path, state) {
        if (isInNodeModules(state)) {
          return;
        }
        console.log("arrow function expression", path.hub.file.opts.filename);
        if (isReactComponent(path)) {
          console.log("is react component");
          // Check to see if this is already wrapped in observer
          if (isWrappedInObserver(path)) {
            console.log("already wrapped in observer");
            return;
          } else {
            console.log("not wrapped in observer");
            // Wrap the ArrowFunctionExpression with observer()
            const observerFunction = t.callExpression(t.identifier("observer"), [path.node]);

            // Replace the ArrowFunctionExpression with the observer() wrapped version
            path.replaceWith(observerFunction);
          }
        } else {
          console.log("not a react component");
        }
      },
      ClassDeclaration(path, state) {
        if (isInNodeModules(state)) {
          return;
        }
        if (isReactComponent(path)) {
          var _path$node$id$name, _path$node$id;
          console.log("hasReactComponent class", (_path$node$id$name = (_path$node$id = path.node.id) === null || _path$node$id === void 0 ? void 0 : _path$node$id.name) !== null && _path$node$id$name !== void 0 ? _path$node$id$name : "Anonymous");
          // Check to see if this is already wrapped in observer
          if (isWrappedInObserver(path)) {
            var _path$node$id$name2, _path$node$id2;
            console.log("already wrapped in observer", (_path$node$id$name2 = (_path$node$id2 = path.node.id) === null || _path$node$id2 === void 0 ? void 0 : _path$node$id2.name) !== null && _path$node$id$name2 !== void 0 ? _path$node$id$name2 : "Anonymous");
            return;
          } else {
            var _path$node$id$name3, _path$node$id3;
            console.log("not wrapped in observer", (_path$node$id$name3 = (_path$node$id3 = path.node.id) === null || _path$node$id3 === void 0 ? void 0 : _path$node$id3.name) !== null && _path$node$id$name3 !== void 0 ? _path$node$id$name3 : "Anonymous");
            wrapClassInObserver(path, t);
          }
        } else {
          var _path$node$id$name4, _path$node$id4;
          console.log("not a react component class", (_path$node$id$name4 = (_path$node$id4 = path.node.id) === null || _path$node$id4 === void 0 ? void 0 : _path$node$id4.name) !== null && _path$node$id$name4 !== void 0 ? _path$node$id$name4 : "Anonymous");
        }
      },
      ClassExpression(path, state) {
        if (isInNodeModules(state)) {
          return;
        }
        if (isReactComponent(path)) {
          console.log("hasReactComponent class expression", path.hub.file.opts.filename);

          // Check to see if this is already wrapped in observer
          if (isWrappedInObserver(path)) {
            var _path$node$id$name5, _path$node$id5;
            console.log("already wrapped in observer", (_path$node$id$name5 = (_path$node$id5 = path.node.id) === null || _path$node$id5 === void 0 ? void 0 : _path$node$id5.name) !== null && _path$node$id$name5 !== void 0 ? _path$node$id$name5 : "Anonymous");
            return;
          } else {
            var _path$node$id$name6, _path$node$id6;
            console.log("not wrapped in observer", (_path$node$id$name6 = (_path$node$id6 = path.node.id) === null || _path$node$id6 === void 0 ? void 0 : _path$node$id6.name) !== null && _path$node$id$name6 !== void 0 ? _path$node$id$name6 : "Anonymous");
            wrapClassInObserver(path, t);
          }
        } else {
          console.log("not a react component class expression", path.hub.file.opts.filename);
        }
      },
      FunctionDeclaration(path, state) {
        if (isInNodeModules(state)) {
          return;
        }
        console.log("checking function declaration", path.node.id.name);
        if (isReactComponent(path)) {
          console.log("hasReactComponent function declaration", path.node.id.name);
          if (isWrappedInObserver(path)) {
            var _path$node$id$name7, _path$node$id7;
            console.log("already wrapped in observer", (_path$node$id$name7 = (_path$node$id7 = path.node.id) === null || _path$node$id7 === void 0 ? void 0 : _path$node$id7.name) !== null && _path$node$id$name7 !== void 0 ? _path$node$id$name7 : "Anonymous");
            return;
          }
          const functionId = path.node.id;

          // Create a FunctionExpression with the same name, params, and body
          const functionExpression = t.functionExpression(functionId, path.node.params, path.node.body, path.node.generator, path.node.async);

          // Wrap the FunctionExpression with observer()
          const observerFunction = t.callExpression(t.identifier("observer"), [functionExpression]);

          // Create a new FunctionDeclaration with the same id and the observer-wrapped body
          const newFunctionDeclaration = t.functionDeclaration(functionId, path.node.params, t.blockStatement([t.returnStatement(observerFunction)]), path.node.generator, path.node.async);

          // Replace the old function declaration with the new one
          path.replaceWith(newFunctionDeclaration);
        } else {
          console.log("not a react component", path.node.id.name);
        }
      },
      FunctionExpression(path, state) {
        var _path$node$id$name8, _path$node$id8;
        if (isInNodeModules(state)) {
          return;
        }
        console.log("checking function expression", (_path$node$id$name8 = (_path$node$id8 = path.node.id) === null || _path$node$id8 === void 0 ? void 0 : _path$node$id8.name) !== null && _path$node$id$name8 !== void 0 ? _path$node$id$name8 : "Anonymous");
        if (isReactComponent(path)) {
          var _path$node$id$name9, _path$node$id9;
          console.log("hasReactComponent function expression", (_path$node$id$name9 = (_path$node$id9 = path.node.id) === null || _path$node$id9 === void 0 ? void 0 : _path$node$id9.name) !== null && _path$node$id$name9 !== void 0 ? _path$node$id$name9 : "Anonymous");

          // Check to see if this is already wrapped in observer
          if (isWrappedInObserver(path)) {
            var _path$node$id$name10, _path$node$id10;
            console.log("already wrapped in observer", (_path$node$id$name10 = (_path$node$id10 = path.node.id) === null || _path$node$id10 === void 0 ? void 0 : _path$node$id10.name) !== null && _path$node$id$name10 !== void 0 ? _path$node$id$name10 : "Anonymous");
            return;
          } else {
            var _path$node$id$name11, _path$node$id11;
            console.log("not wrapped in observer", (_path$node$id$name11 = (_path$node$id11 = path.node.id) === null || _path$node$id11 === void 0 ? void 0 : _path$node$id11.name) !== null && _path$node$id$name11 !== void 0 ? _path$node$id$name11 : "Anonymous");
            // Wrap the FunctionExpression with observer()
            const observerFunction = t.callExpression(t.identifier("observer"), [path.node]);

            // Replace the FunctionExpression with the observer() wrapped version
            path.replaceWith(observerFunction);
          }
        } else {
          var _path$node$id$name12, _path$node$id12;
          console.log("not a react component", (_path$node$id$name12 = (_path$node$id12 = path.node.id) === null || _path$node$id12 === void 0 ? void 0 : _path$node$id12.name) !== null && _path$node$id$name12 !== void 0 ? _path$node$id$name12 : "Anonymous");
        }
      }
    }
  };
};
exports.autoObserverPlugin = autoObserverPlugin;
function isInNodeModules(state) {
  const filename = state.file.opts.filename;
  return filename && filename.includes("node_modules");
}
function isWrappedInObserver(path) {
  return path.parentPath.node.type === "CallExpression" && path.parentPath.node.callee.name === "observer";
}
function wrapClassInObserver(path, t) {
  const classExpression = t.classExpression(path.node.id, path.node.superClass, path.node.body, path.node.decorators);
  // Wrap the ClassDeclaration with observer()
  const observerFunction = t.callExpression(t.identifier("observer"), [classExpression]);

  // Replace the ClassDeclaration with the observer() wrapped version
  path.replaceWith(observerFunction);
}
function isReactComponent(path) {
  console.log("isReactComponent: path node type", path.node.type);
  if (path.node.type === "ArrowFunctionExpression") {
    return doesReturnJSX(path.node.body);
  }
  if (path.node.type === "FunctionDeclaration") {
    return doesReturnJSX(path.node.body);
  }
  if (path.node.type === "FunctionExpression") {
    return doesReturnJSX(path.node.body);
  }
  if (path.node.type === "ClassDeclaration") {
    return classHasRenderMethod(path);
  }
  if (path.node.type === "ClassExpression") {
    return classHasRenderMethod(path);
  }
  return false;
}
function doesReturnJSX(body) {
  if (!body) return false;
  if (body.type === "JSXElement") {
    return true;
  }
  var block = body.body;
  if (block && block.length) {
    var lastBlock = block.slice(0).pop();
    if (lastBlock.type === "ReturnStatement") {
      return lastBlock.argument !== null && lastBlock.argument.type === "JSXElement";
    }
  }
  return false;
}
function classHasRenderMethod(path) {
  if (!path.node.body) {
    return false;
  }
  var members = path.node.body.body;
  for (var i = 0; i < members.length; i++) {
    if (members[i].type === "ClassMethod" && members[i].key.name === "render") {
      return true;
    }
  }
  return false;
}
var _default = exports.default = autoObserverPlugin;
